****************************************************************************
****************************************************************************
**                                                               
**  Mercury (Event Handling)
**
**  This software is in the public domain.  There is no warranty.
**
**  by Patrick Davidson (pad@calc.org, http://pad.calc.org/)
**
**  Last updated August 14, 2002
**
****************************************************************************
****************************************************************************

Handle_Events:
        add.w   d1,d1
        move.w  Event_Table(pc,d1.w),d1
        jmp     Event_Table(pc,d1.w)

EVENT   MACRO
        dc.w    Event_\1-Event_Table
        ENDM

Event_Table:
        EVENT   None
        EVENT   Enter_Artillery         ; 1
        EVENT   Enter_Enemy2_Line       ; 2
        EVENT   Enter_Open_Space        ; 3
        EVENT   Enter_Circling          ; 4
        EVENT   Enter_Base              ; 5
        EVENT   Still_Terrain           ; 6
        EVENT   Lightning_Attack        ; 7
        EVENT   Enter_Boss_1            ; 8
        EVENT   Cross_Formation         ; 9
        EVENT   Heavy_Cannon            ; 10
        EVENT   Single_Loop             ; 11
        EVENT   Enter_Base_Unguarded    ; 12
        EVENT   Top_Bottom_Lightning    ; 13
        EVENT   Sigma_Attack            ; 14
        EVENT   Sine_Attack             ; 15
        EVENT   Enter_Boss_2            ; 16
        EVENT   Wheel                   ; 17

*************************************** WHEEL OF ENEMIES

Event_Wheel:
        lea     \positions(pc),a0
        GETSPR  blob_0,d1               ; D1 = image number
        moveq   #20,d2                  ; D2 = damage
        moveq   #12,d3                  ; D3 = width
        moveq   #12,d4                  ; D4 = height
        move.w  #WHEEL_ENTER,d5         ; D5 = enemy type
        bra     Install_Formation

\positions:
        dc.b    90,45
        dc.b    86,65
        dc.b    73,80
        dc.b    55,89
        dc.b    35,89
        dc.b    17,80
        dc.b    4,65
        dc.b    0,45
        dc.b    4,25
        dc.b    17,10
        dc.b    35,1
        dc.b    55,1
        dc.b    73,10
        dc.b    85,25
        dc.b    18,45
        dc.b    36,45
        dc.b    54,45
        dc.b    72,45
        dc.w    -1

*************************************** SPINNIGN SINE WAVE ENEMIES

Event_Sine_Attack:
        moveq   #5,d0                   ; D0 = number left to insert
        move.w  #208<<4,d1              ; D1 = X coordinate
        move.w  #(64+30)<<4,d2          ; D2 = Y coordinate
        moveq   #0,d3
\l:     bsr     Locate_Empty_Enemy
        bne     Event_None
        move.w  #SINE_SPIN,(a4)
        move.w  d1,e_x(a4)
        add.w   #20<<4,d1
        move.w  d2,e_y(a4)
        add.w   #1<<4,d2
        GETSPR  spin_0a,e_image(a4)
        move.w  #11,e_w(a4)
        move.w  #11,e_h(a4)
        move.w  #10,e_dmg(a4)
        move.w  d3,e_misc(a4)
        add.w   #-122,d3
        dbra    d0,\l
        rts

*************************************** SIGMA ENEMIES IN SIGMA PATTERN

Event_Sigma_Attack:
        lea     \positions(pc),a0
        GETSPR  enemy_sigma,d1          ; D1 = image number
        moveq   #20,d2                  ; D2 = damage
        moveq   #12,d3                  ; D3 = width
        moveq   #16,d4                  ; D4 = height
        move.w  #ENEMY_4_ENTERING,d5    ; D5 = enemy type
        bra     Install_Formation

\positions:
        dc.b    0,0
        dc.b    18,0
        dc.b    36,0
        dc.b    54,0
        dc.b    72,0
        dc.b    90,0
        dc.b    67,67
        dc.b    45,45
        dc.b    67,23
        dc.b    0,90
        dc.b    18,90
        dc.b    36,90
        dc.b    54,90
        dc.b    72,90
        dc.b    90,90
        dc.w    -1

*************************************** LIGHTNING ATTACK ON TOP/BOTTOM

Event_Top_Bottom_Lightning:
        lea     \positions(pc),a0
        bra     Put_Lightning_Attack

\positions:
        dc.w    208<<4,65<<4
        dc.w    208<<4,175<<4
        dc.w    0

*************************************** ENEMIES THAT FOLLOW SINGLE LOOP

Event_Single_Loop:
        moveq   #5,d0                   ; D0 = number left to insert
        move.w  #208<<4,d1              ; D1 = X coordinate
        move.w  #(64+30)<<4,d2          ; D2 = Y coordinate
\l:     bsr     Locate_Empty_Enemy
        bne     Event_None
        move.w  #SINGLE_LOOP_ENTER,(a4)
        move.w  d1,e_x(a4)
        add.w   #30<<4,d1
        move.w  d2,e_y(a4)
        add.w   #1<<4,d2
        GETSPR  spin_0a,e_image(a4)
        move.w  #11,e_w(a4)
        move.w  #11,e_h(a4)
        move.w  #10,e_dmg(a4)
        dbra    d0,\l
        rts

*************************************** HEAVY CANNON

Event_Heavy_Cannon:
        move.w  #(64+128+6)<<4,d5
        bsr     Get_Level_Width
\l:     sub.w   d6,a1
        sub.w   #16<<4,d5
        cmp.b   #30,(a1)
        bne.s   \l

        bsr     Locate_Empty_Enemy
        bne.s   \done
        move.w  #HEAVY_CANNON,(a4)
        move.w  d5,e_y(a4)
        move.w  #208<<4,e_x(a4)
        move.w  #30,e_dmg(a4)
        move.w  #12,e_h(a4)
        move.w  #17,e_w(a4)
        GETSPR  cannon,e_image(a4)
\done:  rts

*************************************** CROSS FORMATION

Event_Cross_Formation:
        lea     cross_positions(pc),a0
        GETSPR  enemy_4,d1              ; D1 = image number
        moveq   #8,d2                   ; D2 = damage
        moveq   #12,d3                  ; D3 = width
        moveq   #10,d4                  ; D4 = height
        move.w  #ENEMY_4_ENTERING,d5    ; D5 = enemy type

Install_Formation:
        bsr     Locate_Empty_Enemy
        bne.s   \done
        clr.w   d0
        move.b  (a0)+,d0
        blt.s   \done
        move.w  d0,e_timer(a4)
        lsl.w   #4,d0
        add.w   #208<<4,d0
        move.w  d0,e_x(a4)
        clr.w   d0
        move.b  (a0)+,d0
        add.w   #79,d0
        lsl.w   #4,d0
        move.w  d0,e_y2(a4)
        move.w  d5,(a4)
        move.w  d1,e_image(a4)
        move.w  d2,e_dmg(a4)
        move.w  d3,e_w(a4)
        move.w  d4,e_h(a4)
        move.w  #-70,e_misc(a4)
        bra.s   Install_Formation
\done:  rts

cross_positions:
        dc.b    0,0
        dc.b    45,0
        dc.b    45,15
        dc.b    45,30
        dc.b    45,45
        dc.b    45,60
        dc.b    45,75
        dc.b    45,90 
        dc.b    0,45
        dc.b    15,45
        dc.b    30,45
        dc.b    60,45
        dc.b    75,45
        dc.b    90,45
        dc.b    0,90
        dc.b    90,0
        dc.b    90,90
        dc.w    -1

*************************************** FIRST TYPE OF BOSS

Event_Enter_Boss_1:
        bsr     Locate_Empty_Enemy
        bne.s   \quit
        move.w  #BOSS1,(a4)
        move.w  #208<<4,e_x(a4)
        move.w  #70<<4,e_y(a4)
        move.w  #16,e_misc(a4)
        move.w  #60,e_dmg(a4)
        move.w  #17,e_w(a4)
        move.w  #27,e_h(a4)
        GETSPR  boss_1,e_image(a4)
\quit:  rts

*************************************** FIRST TYPE OF BOSS

Event_Enter_Boss_2:
        bsr     Locate_Empty_Enemy
        bne.s   \quit
        move.w  #BOSS2,(a4)
        move.w  #208<<4,e_x(a4)
        move.w  #70<<4,e_y(a4)
        move.w  #16,e_misc(a4)
        move.w  #300,e_dmg(a4)
        move.w  #17,e_w(a4)
        move.w  #42,e_h(a4)
        GETSPR  boss_2,e_image(a4)
\quit:  rts

*************************************** ENEMY 3 IN ZIG-ZAG PATTERNS

Event_Lightning_Attack:
        lea     lightning_positions(pc),a0
Put_Lightning_Attack:
\il:    bsr     Locate_Empty_Enemy
        bne.s   \done
        move.w  (a0)+,e_x(a4)
        beq.s   \done
        move.w  (a0)+,e_y(a4)
        move.w  #ENEMY_3,(a4)
        GETSPR  enemy_3,e_image(a4)
        move.w  #4,e_dmg(a4)
        move.w  #16,e_w(a4)
        move.w  #12,e_h(a4)
        clr.w   e_timer(a4)
        clr.w   e_path(a4)
        bra.s   \il
\done:  rts

lightning_positions:
        dc.w    208<<4,70<<4
        dc.w    208<<4,90<<4
        dc.w    208<<4,110<<4
        dc.w    208<<4,130<<4
        dc.w    240<<4,80<<4
        dc.w    240<<4,100<<4
        dc.w    240<<4,120<<4
        dc.w    0
        
*************************************** STOP TERRAIN MOTION

Event_Still_Terrain:
        st      open_space+1(a5)
        rts

*************************************** INSERT SMALL BASE AND 4 GUARDS

Event_Enter_Base_Unguarded:
Event_Enter_Base:
        move.b  (a1),d2
        move.w  #(64+128-2)<<4,d5
        bsr     Get_Level_Width
\ll:    sub.w   d6,a1
        sub.w   #16<<4,d5
        cmp.b   #29,(a1)
        bne.s   \ll

        bsr     Locate_Empty_Enemy      ; add base
        bne.s   \q                   
        move.w  #BASE,(a4)
        move.w  #208<<4,e_x(a4)
        move.w  d5,e_y(a4)
        GETSPR  base_0,e_image(a4)
        clr.w   e_misc(a4)
        move.w  #17,e_w(a4)
        move.w  #20,e_dmg(a4)
        move.w  #18,e_h(a4)

        cmp.b   #12,d2
        beq.s   \q

        add.w   #4<<4,d5
        moveq   #0,d0                   ; add guards
        moveq   #3,d1
\l:     bsr     Locate_Empty_Enemy
        bne.s   \q
        move.w  #GUARD_ENTRY,(a4)
        move.w  d0,e_y(a4)
        sub.w   #64<<4,d0
        move.w  d5,e_y2(a4)
        move.w  #179<<4,e_x(a4)
        GETSPR  spin_0a,e_image(a4)
        move.w  #11,e_w(a4)
        move.w  #5,e_dmg(a4)
        move.w  #11,e_h(a4)
        clr.w   e_misc(a4)
        dbra    d1,\l

\q:     rts

*************************************** INSERT WOBBLING OVAL OF ENEMIES

Event_Enter_Circling:
        moveq   #18,d5                  ; D7 = # enemies remaining
        moveq   #0,d6                   ; D0 = angle in ellipse
\loop_enter_circle:
        bsr     Locate_Empty_Enemy
        bne.s   Event_None

        move.w  d6,d0
        add.w   #27,d6
        bsr     sin_cos
        ext.l   d0
        divs    #5,d0
        add.w   #(122)<<4,d0
        move.w  d0,e_y2(a4)
        ext.l   d1
        divs    #9,d1
        add.w   #180<<4,d1
        move.w  d1,e_x2(a4)

        move.w  #CIRCLING_ENTRY,(a4)
        GETSPR  e_small_0,e_image(a4)
        clr.w   e_misc(a4)
        move.w  #10,e_w(a4)
        move.w  #1,e_dmg(a4)
        move.w  #10,e_h(a4)
        dbra    d5,\loop_enter_circle
        rts

**************************************** ENTER OPEN SPACE GAME MODE

Event_Enter_Open_Space:
        st      open_space(a5)
        rts

**********************8***************** DO NOTHING

Event_None:
        rts

**************************************** INSERT A TANK

Event_Enter_Artillery:
        bsr     Locate_Empty_Enemy
        bne.s   Event_None
        move.w  #ARTILLERY,(a4)
        move.w  #208<<4,e_x(a4)
        move.w  #(64+100)<<4,e_y(a4)
        GETSPR  artillery_0,e_image(a4)
        clr.w   e_misc(a4)
        move.w  #17,e_w(a4)
        move.w  #2,e_dmg(a4)
        move.w  #13,e_h(a4)
        rts

**************************************** INSERT LINE OF SWOOPING ENEMY 2

Event_Enter_Enemy2_Line:
        moveq   #3,d0                   ; D0 = number left to insert
        move.w  #208<<4,d1              ; D1 = X coordinate
        move.w  #(64+30)<<4,d2          ; D2 = Y coordinate
\l:     bsr     Locate_Empty_Enemy
        bne.s   Event_None
        move.w  #ENEMY_2,(a4)
        move.w  d1,e_x(a4)
        add.w   #30<<4,d1
        move.w  d2,e_y(a4)
        add.w   #13<<4,d2
        GETSPR  enemy_2,e_image(a4)
        move.w  #15,e_w(a4)
        move.w  #10,e_h(a4)
        move.w  #1,e_dmg(a4)
        move.w  #-16,e_misc(a4)
        dbra    d0,\l
        rts
